/****************************************************************
 * a_adj.c
 * Last change 12-22-2013 by Osman Celimli
 ****************************************************************/
#define _CRTDBG_MAP_ALLOC
#include <stdlib.h>
#include <crtdbg.h>

#include <stdio.h>
#include <string.h>
#include <windows.h>

#include "useful.h"
#include "options.h"
#include "wavemath.h"
/****************************************************************
 * Allocation Settings
 ****************************************************************/
#define	INIT_LINEBUF_SIZE	256 * sizeof(char)
#define	INIT_SWEEPBUF_SIZE	1024 * sizeof(float)
/****************************************************************
 * Important Globals
 ****************************************************************/
int iAdjusting = 0;
unsigned char aucAdjustTable[256];
/****************************************************************
 * Text Tables
 ****************************************************************/
char ERR_NoAdjMem[]= "ERROR- Out of memory\n";

char ERR_NoOpenAdj[]= "ERROR- Can't open csv sweep : ";
char ERR_NoParseAmp[]= "ERROR- Can't parse amplitude : ";

char ERR_SmallSweep[]= "ERROR- Inadequate sweep length, required : ";
char ERR_MangledSweep[]= "ERROR- Mangled sweep, tried to match : ";

char szWhiteSpace[]=" ,\t\n";
/****************************************************************
 * int a_genAdjTable(char *pszSweepFile)
 * Generate the adjustment table (for linear to nonlinear audio
 * conversions) based upon a supplied single column csv file.
 ****************************************************************/
int a_genAdjTable(char *pszSweepFile)
{
	FILE *pAdjFile = NULL;
	char *pszLineBuf = NULL;
	unsigned int uiLineBufLen;

	float *pafRawSweep = NULL;
	unsigned int uiSweepLen,uiSweepBufLen;

	char *pszCurTok;
	unsigned int uiX,uiY;
	int iRet,iScanRes;
	float fScalar,fCurAmp,fCurThresh;

	unsigned int uiMangle;
	float fMangle;

	/**
	 * Attempt to open our source file then allocate our
	 * line and sweep buffers.
	 */
	iRet = -1;
	if(NULL == (pAdjFile = fopen(pszSweepFile,"r")))
	{
		fprintf(stderr,"%s%s\n", ERR_NoOpenAdj, pszSweepFile);
		return -1;
	}
	pszLineBuf = malloc(INIT_LINEBUF_SIZE);
	uiLineBufLen = INIT_LINEBUF_SIZE;

	pafRawSweep = malloc(INIT_SWEEPBUF_SIZE);
	uiSweepBufLen = INIT_SWEEPBUF_SIZE;
	uiSweepLen = 0;

	if((pszLineBuf == NULL) || (pafRawSweep == NULL))
	{
		fprintf(stderr, "%s", ERR_NoAdjMem);
		goto cleanup;
	}

	/**
	 * Read in the first column of the csv file, then
	 * check to make sure we've actually been provided
	 * with a useful quantity of data.
	 */
	for(uiX = 0;; uiX++)
	{
		if(fgets(pszLineBuf, INIT_LINEBUF_SIZE, pAdjFile) == NULL) break;

		pszCurTok = strtok(pszLineBuf, szWhiteSpace);
		if(pszCurTok != NULL)
		{
			iScanRes = sscanf(pszCurTok, "%f", &fCurAmp);
			if(iScanRes < 1)
			{
				fprintf(stderr, "%s%s in %s,%u", ERR_NoParseAmp,
					pszCurTok, pszSweepFile, (uiX + 1));
			}

			uiSweepLen++;
			if(tealloc(&pafRawSweep, &uiSweepBufLen, (uiSweepLen * sizeof(float))))
			{
				fprintf(stderr, "%s", ERR_NoAdjMem);
				goto cleanup;
			}
			pafRawSweep[uiX] = fCurAmp;
		}
	}
	if(uiSweepLen < 16)
	{

		goto cleanup;
	}

	/**
	 * Adjust the the read amplitude sweep so that its
	 * minimum is at zero and its maximum is at 255.
	 */
	if(wave_adjScale(pafRawSweep, uiSweepLen, 0.0, 255.0))
	{
		fprintf(stderr, "%s%u, had : %u", ERR_SmallSweep, 16, uiSweepLen);
		goto cleanup;
	}

	/**
	 * Traverse through the adjusted signal and try to find each of the
	 * possible amplitudes of our 8-Bit DAC. If all goes well, our
	 * adjustment table is then ready for use.
	 */
	fScalar = ((float)256) / ((float)uiSweepLen);
	for(uiX = 0; uiX < 256; uiX++)
	{
		fCurAmp = (float)uiX;
		fCurThresh = 0.5;

		for(;;)
		{
			if(wave_findVal(pafRawSweep, uiSweepLen, fCurAmp, fCurThresh, &uiY))
			{
				fMangle = ((float)uiY) * fScalar;
				if(fMangle > 255.0) fMangle = 255.0;
				if(fMangle < 0.0) fMangle = 0.0;

				uiMangle = (unsigned int)fMangle;
				aucAdjustTable[uiX] = uiMangle;
				break;
			}
			else if(fCurThresh > 128.0)
			{
				fprintf(stderr, "%s%u, last threshold %f", ERR_MangledSweep,
					uiX, fCurThresh);
				goto cleanup;
			}
			fCurThresh *= 2.0;
		}

	}
	iAdjusting = -1;
	iRet = 0;

cleanup:
	if(pszLineBuf != NULL) free(pszLineBuf);
	if(pafRawSweep != NULL) free(pafRawSweep);
	fclose(pAdjFile);
	return iRet;
}
/****************************************************************
 * unsigned char a_adjVoice(unsigned char ucByte)
 * Adjust the given byte of data using our generated adjustment
 * table (if enabled), otherwise just return it as-is.
 ****************************************************************/
unsigned char a_adjVoice(unsigned char ucByte)
{
	if(iAdjusting) return aucAdjustTable[ucByte];
	else return ucByte;
}
