/****************************************************************
 * b_bits.c
 * Last change 12-22-2013 by Osman Celimli
 ****************************************************************/
#define _CRTDBG_MAP_ALLOC
#include <stdlib.h>
#include <crtdbg.h>

#include <stdio.h>
#include <string.h>
#include <windows.h>

#include "useful.h"
#include "options.h"
/****************************************************************
 * Important Globals
 ****************************************************************/
int iBitBobbling = 0;

char szBitTable[]= "76543210";
char szBitDefault[]= "76543210";
unsigned char aucByteBits[]={0x01,0x02,0x04,0x08,0x10,0x20,0x40,0x80};
/****************************************************************
 * Text Tables
 ****************************************************************/
char ERR_CountBits[]= "ERROR- Bit arrangement not one byte long : ";
char ERR_BadBits[]= "ERROR- Invalid entry in bit arrangement : ";
/****************************************************************
 * int b_setBitTable(char *pszNewBits)
 * Set up the bit arrangement table using the supplied string.
 ****************************************************************/
int b_setBitTable(char *pszNewBits)
{
	unsigned int uiX;

	/**
	 * Ensure the bit table is of the format '76543210', repeats
	 * are allowed in order to support bit-doubling.
	 */
	if(strlen(pszNewBits) != 8)
	{
		fprintf(stderr, "%s%s\n", ERR_CountBits, pszNewBits);
		return -1;
	}
	for(uiX = 0; uiX < 8; uiX++)
	{
		if((pszNewBits[uiX] < '0') || (pszNewBits[uiX] > '7'))
		{
			fprintf(stderr, "%s%s\n", ERR_BadBits, pszNewBits);
			return -1;
		}
	}

	/**
	 * Looks like the arrangement table is fine,
	 * copy it over the old one. But make sure that we're
	 * actually doing a transform before setting the
	 * bobble enable flag.
	 */
	strcpy(szBitTable, pszNewBits);
	iBitBobbling = strcmp(szBitTable, szBitDefault);
	return 0;
}
/****************************************************************
 * unsigned char b_bitBobble(unsigned char ucByte)
 * Adjust the bits of the given byte according to our
 * arrangement table.
 ****************************************************************/
unsigned char b_bitBobble(unsigned char ucByte)
{
	unsigned int uiX,uiY;
	unsigned char ucBobble;
	char cCurBit;

	if(iBitBobbling)
	{
		ucBobble = 0;
		for(uiX = 0; uiX < 8; uiX++)
		{
			if(ucByte & aucByteBits[uiX])
			{
				for(uiY = 0; uiY < 8; uiY++)
				{
					cCurBit = uiX + '0';
					if(szBitTable[uiY] == cCurBit)
					{
						ucBobble |= aucByteBits[7 - uiY];
					}
				}
			}
		}
	}
	else
	{
		ucBobble = ucByte;
	}
	return ucBobble;
}
